/*
# Errors

Strata provides a clean abstraction for errors that makes it simple to build
meaningful hierarchies of errors, with support for nesting. You can inherit from
Strata's base error prototype, `strata.errors.Error` to get this extra
functionality.

    var util = require("util"),
        strata = require("strata"),
        errors = strata.errors;

    function LoginFailedError(message, cause) {
        message = message || "Failed Login";
        errors.Error.call(this, message, cause);
    }

    util.inherits(InvalidLoginError, errors.Error);

    function InvalidLoginError(message) {

    }

    util.inherits(InvalidLoginError, LoginFailedError);

And your app might look something like this:

    function app(env, callback) {
        var req = new Request(env);

        req.params(function (err, params) {
            if (err && strata.handleError(err, env, callback)) {
                return true;
            }

            var username = params.username;
            var password = params.password;

            if (validUsernameAndPassword(username, password)) {

            } else {
                callback(403, {}, "Invalid username and/or password!");
            }
        });
    }

## Error Handling

If you read through the example code in the previous chapter, you may have
noticed that there was an `err` argument provided in the callback to
`req.params` that was simply ignored. This style of passing errors as the
first argument to a callback is standard practice in node programs.

Typically, error handling in an asynchronous environment is a pain point for
programmers. The problem is that when you're in a callback you can't simply
`throw` the error because the callback is called out of the original calling
context. Thus, any `try`/`catch` mechanism that was used when the caller
was called is not able to catch the error when thrown. Since an uncaught
error will crash the program, we need a mechanism to handle them gracefully.

The solution in Strata is the global error handler, `strata.handleError`.
This function takes three arguments: the error, and the environment and
callback arguments that were passed to the app. In Strata, the environment
and callback represent the context of the request, so even though the calling
context is no longer accessible we can still return an intelligent response
to the client without bringing the whole server down.

The `strata.handleError` function handles the error by issuing a 500 Internal
Server Error to the client and writing the error to `env.error` (the error
stream, see the [SPEC](https://github.com/mjijackson/strata/blob/master/SPEC)).
This function should be overridden by users who wish to provide custom error
handling logic at the global level.

For example, you may wish to serve a 400 response to requests whose body is not
consistent with the encoding specified in the Content-Type header. You could do
this by overriding `strata.errorHandler` like this:

    var strata = require("strata"),
        errors = strata.errors;

    // Perserve a reference to the original error handler.
    var _handleError = strata.handleError;

    strata.handleError = function (err, env, callback) {
        if (err instanceof errors.InvalidRequestBodyError) {
            callback(400, {}, "Invalid Request Body");
            return true;
        }

        return _handleError(err, env, callback);
    }

Another use case might be if your database driver returns an error when you run
a query using a primary key that is not present in the database. In this
case, you could setup a global error handler that handles this specific type
of error by returning a 404 to the client, instead of returning a 500.

The return value of `strata.handleError` **must** always be a boolean. A `true`
value means that the callback was used to issue a response to the client so
the server may stop processing the request, while `false` means that the error
was not fatal and the request may continue processing. By default all errors are
considered fatal, so `true` is returned.

The following app is a clone of the example in the previous chapter, except this
time any error that may be returned in the callback to `req.params` will be
handled gracefully.
*/

var strata = require("strata"),
    Request = strata.Request;

module.exports = function (env, callback) {
    var req = new Request(env);

    req.params(function (err, params) {
        // If there was an error and strata.handleError issued a response based
        // on it, return and stop processing the request.
        if (err && strata.handleError(err, env, callback)) {
            return;
        }

        var content = JSON.stringify(params);

        callback(200, {
            "Content-Type": "application/json",
            "Content-Length": Buffer.byteLength(content).toString()
        }, content);
    });
}

/*
In this example, an error returned by `req.params` represents an error that
Strata's request parser encountered when parsing the request query or body.

As in previous chapters, you can save the above code to a file named `app.js`
and run it with:

    $ strata app.js

Then view the app at [http://localhost:1982/](http://localhost:1982/). To get
this app to return a 500 response, send it some bad data. For example, send an
`application/x-www-url-formencoded` request with incomplete data in the request
body, like this:

    $ curl -v -H "Content-Type: application/json" --data "a" http://localhost:1982/

When the JSON parser is unable to parse the request body, it will return an
error to the call to `req.params`, which in turn will issue a 500 response using
`strata.handleError`.
*/
